@@ -3,13 +3,12 @@ package Business::ES::NIF;
 # ABSTRACT: Validate Spanish NIF, NIE and CIF numbers
 
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 use strict;
 use warnings;
 
-
-use Class::XSAccessor { accessors => [qw(error extra nif status type vies_check vies_error)] };
+use Class::XSAccessor { accessors => [qw(error extra nif status type provincia vies_check vies_error)] };
 
 
 use constant {
@@ -22,106 +21,134 @@ use constant {
 	      RE_NIE     => qr/^([XYZ])([0-9]{7})([A-Z])$/
 	     };
 
-my %PROVINCIAS = (
-		  '00' => 'No Residente',
-		  '01' => 'Álava',
-		  '02' => 'Albacete', 
-		  '03' => 'Alicante',
-		  '04' => 'Almería',
-		  '05' => 'Ávila',
-		  '06' => 'Badajoz',
-		  '07' => 'Islas Baleares',
-		  '08' => 'Barcelona',
-		  '09' => 'Burgos',
-		  '10' => 'Cáceres',
-		  '11' => 'Cádiz',
-		  '12' => 'Castellón',
-		  '13' => 'Ciudad Real',
-		  '14' => 'Córdoba',
-		  '15' => 'La Coruña',
-		  '16' => 'Cuenca',
-		  '17' => 'Gerona',
-		  '18' => 'Granada',
-		  '19' => 'Guadalajara',
-		  '20' => 'Guipúzcoa',
-		  '21' => 'Huelva',
-		  '22' => 'Huesca',
-		  '23' => 'Jaén',
-		  '24' => 'León',
-		  '25' => 'Lérida',
-		  '26' => 'La Rioja',
-		  '27' => 'Lugo',
-		  '28' => 'Madrid',
-		  '29' => 'Málaga',
-		  '30' => 'Murcia',
-		  '31' => 'Navarra',
-		  '32' => 'Orense',
-		  '33' => 'Asturias',
-		  '34' => 'Palencia',
-		  '35' => 'Las Palmas',
-		  '36' => 'Pontevedra',
-		  '37' => 'Salamanca',
-		  '38' => 'Santa Cruz de Tenerife',
-		  '39' => 'Cantabria',
-		  '40' => 'Segovia',
-		  '41' => 'Sevilla',
-		  '42' => 'Soria',
-		  '43' => 'Tarragona',
-		  '44' => 'Teruel',
-		  '45' => 'Toledo',
-		  '46' => 'Valencia',
-		  '47' => 'Valladolid',
-		  '48' => 'Vizcaya',
-		  '49' => 'Zamora',
-		  '50' => 'Zaragoza',
-		  '51' => 'Ceuta',
-		  '52' => 'Melilla',
-		  '53' => 'Alicante',         
-		  '54' => 'Alicante',         
-		  '55' => 'Gerona',           
-		  '56' => 'Córdoba',          
-		  '57' => 'Islas Baleares',   
-		  '58' => 'Barcelona',        
-		  '59' => 'Barcelona',
-		  '60' => 'Barcelona',
-		  '61' => 'Barcelona',
-		  '62' => 'Barcelona',
-		  '63' => 'Barcelona',
-		  '64' => 'Barcelona',
-		  '65' => 'Barcelona',
-		  '66' => 'Barcelona',
-		  '67' => 'Gerona',           
-		  '68' => 'Barcelona',        
-		  '70' => 'La Coruña',        
-		  '71' => 'Guipúzcoa',        
-		  '72' => 'Cádiz',            
-		  '73' => 'Murcia',           
-		  '74' => 'Asturias',         
-		  '75' => 'Las Palmas',       
-		  '76' => 'Santa Cruz de Tenerife',
-		  '77' => 'Tarragona',        
-		  '78' => 'Madrid',           
-		  '79' => 'Madrid',
-		  '80' => 'Madrid',
-		  '81' => 'Madrid',
-		  '82' => 'Madrid',
-		  '83' => 'Madrid',
-		  '84' => 'Madrid',
-		  '85' => 'Madrid',
-		  '86' => 'Madrid',
-		  '87' => 'Madrid',
-		  '88' => 'Madrid',
-		  '90' => 'Sevilla',          
-		  '91' => 'Sevilla',          
-		  '92' => 'Málaga',           
-		  '93' => 'Málaga',           
-		  '94' => 'Pontevedra',       
-		  '95' => 'Vizcaya',          
-		  '96' => 'Valencia',         
-		  '97' => 'Valencia',
-		  '98' => 'Valencia',
-		  '99' => 'Zaragoza',
-		 );
+use constant NIE_PREFIX => { X => '0', Y => '1', Z => '2' };
+
+use constant CIF_EXTRA => {
+			   'A' => 'Sociedad Anonima - S.A',
+			   'B' => 'Sociedad Limitada - S.L',
+			   'C' => 'Sociedad Colectiva - S.C',
+			   'D' => 'Sociedades comanditarias',
+			   'E' => 'Comunidad de bienes y herencias',
+			   'F' => 'Sociedades cooperativas',
+			   'G' => 'Asociaciones',
+			   'H' => 'Comunidaddes de propietarios',
+			   'J' => 'Sociedades civiles',
+			   'P' => 'Corporaciones locales',
+			   'Q' => 'Organismos publicos',
+			   'N' => 'Entidades extranjeras',
+			   'R' => 'Congregaciones e instituciones religiosas',
+			   'U' => 'Uniones temporales de epresas',
+			   'V' => 'Otros tipos de sociedades',
+			   'W' => 'Establecimientos permanentes de entidades no residentes en España'
+			  };
+
+use constant CIFe_EXTRA => {
+			    'S' => 'Órganos de la Administración del Estado',
+			    'Q' => 'Organismos autónomos o instituciones religiosas',
+			    'P' => 'Corporaciones locales',
+			    'K' => 'Formato antiguo (DNI sin letra de control)'
+			   };
+
+use constant PROVINCIAS => {
+			    '00' => 'No Residente',
+			    '01' => 'Álava',
+			    '02' => 'Albacete', 
+			    '03' => 'Alicante',
+			    '04' => 'Almería',
+			    '05' => 'Ávila',
+			    '06' => 'Badajoz',
+			    '07' => 'Islas Baleares',
+			    '08' => 'Barcelona',
+			    '09' => 'Burgos',
+			    '10' => 'Cáceres',
+			    '11' => 'Cádiz',
+			    '12' => 'Castellón',
+			    '13' => 'Ciudad Real',
+			    '14' => 'Córdoba',
+			    '15' => 'La Coruña',
+			    '16' => 'Cuenca',
+			    '17' => 'Gerona',
+			    '18' => 'Granada',
+			    '19' => 'Guadalajara',
+			    '20' => 'Guipúzcoa',
+			    '21' => 'Huelva',
+			    '22' => 'Huesca',
+			    '23' => 'Jaén',
+			    '24' => 'León',
+			    '25' => 'Lérida',
+			    '26' => 'La Rioja',
+			    '27' => 'Lugo',
+			    '28' => 'Madrid',
+			    '29' => 'Málaga',
+			    '30' => 'Murcia',
+			    '31' => 'Navarra',
+			    '32' => 'Orense',
+			    '33' => 'Asturias',
+			    '34' => 'Palencia',
+			    '35' => 'Las Palmas',
+			    '36' => 'Pontevedra',
+			    '37' => 'Salamanca',
+			    '38' => 'Santa Cruz de Tenerife',
+			    '39' => 'Cantabria',
+			    '40' => 'Segovia',
+			    '41' => 'Sevilla',
+			    '42' => 'Soria',
+			    '43' => 'Tarragona',
+			    '44' => 'Teruel',
+			    '45' => 'Toledo',
+			    '46' => 'Valencia',
+			    '47' => 'Valladolid',
+			    '48' => 'Vizcaya',
+			    '49' => 'Zamora',
+			    '50' => 'Zaragoza',
+			    '51' => 'Ceuta',
+			    '52' => 'Melilla',
+			    '53' => 'Alicante',         
+			    '54' => 'Alicante',         
+			    '55' => 'Gerona',           
+			    '56' => 'Córdoba',          
+			    '57' => 'Islas Baleares',   
+			    '58' => 'Barcelona',        
+			    '59' => 'Barcelona',
+			    '60' => 'Barcelona',
+			    '61' => 'Barcelona',
+			    '62' => 'Barcelona',
+			    '63' => 'Barcelona',
+			    '64' => 'Barcelona',
+			    '65' => 'Barcelona',
+			    '66' => 'Barcelona',
+			    '67' => 'Gerona',           
+			    '68' => 'Barcelona',        
+			    '70' => 'La Coruña',        
+			    '71' => 'Guipúzcoa',        
+			    '72' => 'Cádiz',            
+			    '73' => 'Murcia',           
+			    '74' => 'Asturias',         
+			    '75' => 'Las Palmas',       
+			    '76' => 'Santa Cruz de Tenerife',
+			    '77' => 'Tarragona',        
+			    '78' => 'Madrid',           
+			    '79' => 'Madrid',
+			    '80' => 'Madrid',
+			    '81' => 'Madrid',
+			    '82' => 'Madrid',
+			    '83' => 'Madrid',
+			    '84' => 'Madrid',
+			    '85' => 'Madrid',
+			    '86' => 'Madrid',
+			    '87' => 'Madrid',
+			    '88' => 'Madrid',
+			    '90' => 'Sevilla',          
+			    '91' => 'Sevilla',          
+			    '92' => 'Málaga',           
+			    '93' => 'Málaga',           
+			    '94' => 'Pontevedra',       
+			    '95' => 'Vizcaya',          
+			    '96' => 'Valencia',         
+			    '97' => 'Valencia',
+			    '98' => 'Valencia',
+			    '99' => 'Zaragoza'
+			   };
 
 
 
@@ -134,7 +161,7 @@ sub is_cife { return shift->type eq 'CIFe' }
 sub _provincia {
   my $codigo = shift;
 
-  return $PROVINCIAS{$codigo} if $codigo =~ /^[0-9]{2}$/;
+  return PROVINCIAS->{$codigo} if $codigo =~ /^[0-9]{2}$/;
   return undef;
 }
 
@@ -153,12 +180,10 @@ sub _check_cif {
   my $c = (10 - substr($pares + $nones, -1)) % 10;
   my $l = substr(CIF_LETRAS, $c, 1);
   
-  for ($sociedad) {
-    if (/[KPQS]/i) {
-      return 0 if $l ne uc($control);
-    }else {
-      return 0 if $c != $control and $l ne uc($control);
-    }
+  if ( $sociedad =~ /[KPQS]/i) {
+    return 0 if $l ne uc($control);
+  } else {
+    return 0 if $c != $control and $l ne uc($control);
   }
     
   return 1;
@@ -168,27 +193,10 @@ sub _validate_cif {
   my ($self,$sociedad, $inscripcion, $control) = @_;
 
   $self->status(_check_cif($sociedad, $inscripcion, $control));
-  $self->{provincia} = _provincia(substr($inscripcion,0,2));
+  $self->provincia(_provincia(substr($inscripcion,0,2)));
 
   $self->type('CIF');
-  $self->extra({
-		'A' => 'Sociedad Anonima - S.A',
-		'B' => 'Sociedad Limitada - S.L',
-		'C' => 'Sociedad Colectiva - S.C',
-		'D' => 'Sociedades comanditarias',
-		'E' => 'Comunidad de bienes y herencias',
-		'F' => 'Sociedades cooperativas',
-		'G' => 'Asociaciones',
-		'H' => 'Comunidaddes de propietarios',
-		'J' => 'Sociedades civiles',
-		'P' => 'Corporaciones locales',
-		'Q' => 'Organismos publicos',
-		'N' => 'Entidades extranjeras',
-		'R' => 'Congregaciones e instituciones religiosas',
-		'U' => 'Uniones temporales de epresas',
-		'V' => 'Otros tipos de sociedades',
-		'W' => 'Establecimientos permanentes de entidades no residentes en España',
-	       }->{ $sociedad } );
+  $self->extra( CIF_EXTRA->{ $sociedad } );
 }
 
 sub _validate_cife {
@@ -197,12 +205,7 @@ sub _validate_cife {
   $self->status(_check_cif($sociedad, $inscripcion, $control));
 
   $self->type('CIFe');
-  $self->extra({
-	       'S' => 'Órganos de la Administración del Estado',
-	       'Q' => 'Organismos autónomos o instituciones religiosas',
-	       'P' => 'Corporaciones locales',
-	       'K' => 'Formato antiguo (DNI sin letra de control)',
-	      }->{ $sociedad });
+  $self->extra( CIFe_EXTRA->{ $sociedad } );
 }
 
 sub _check_nif {
@@ -232,9 +235,7 @@ sub _validate_nife {
 sub _validate_nie {
   my ($self,$NIE,$NIF,$DC) = @_;
 
-  if    ($NIE eq 'X') { $NIF = '0'.$NIF; }
-  elsif ($NIE eq 'Y') { $NIF = '1'.$NIF; }
-  elsif ($NIE eq 'Z') { $NIF = '2'.$NIF; }
+  $NIF = NIE_PREFIX->{$NIE} . $NIF;
 
   $self->status(_check_nif($NIF,$DC));
 
@@ -246,7 +247,7 @@ sub _validate_nie {
 sub new {
   my ($class, %args) = @_;
 
-  my $self = { vies   => $args{vies} // 0 };
+  my $self = { vies => $args{vies} // 0 };
   
   $self = bless $self, $class;
 
@@ -276,7 +277,7 @@ sub set {
 }
 
 
-sub is_valid { return shift->status ? 1 : 0; }
+sub is_valid { !!shift->status }
 
 
 sub check {
@@ -355,7 +356,7 @@ Business::ES::NIF - Validate Spanish NIF, NIE and CIF numbers
 
 =head1 VERSION
 
-version 0.12
+version 0.13
 
 =head1 SYNOPSIS
 
@@ -406,6 +407,10 @@ Se puede activar la comprobacion sobre el VIES ( Business::Tax::VAT::Validation
 
 =head2 status
 
+=head2 provincia
+
+my $city = $obj->provincia;
+
 =head2 vies_check
 
 =head2 vies_error
